package ru.dbotthepony.mc.otm.android;

import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.registries.ForgeRegistry;
import net.minecraftforge.registries.RegistryManager;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.android.AndroidResearchRequestPacket;
import ru.dbotthepony.mc.otm.screen.SkinElement;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;

public abstract class AndroidResearch implements INBTSerializable<CompoundTag> {
	public final AndroidResearchType<?> type;
	public final AndroidCapabilityPlayer capability;

	protected boolean researched = false;
	public boolean dirty = true;

	public void markDirty() {
		dirty = true;
	}

	public boolean isResearched() {
		return researched;
	}

	public AndroidResearch(AndroidResearchType<?> type, AndroidCapabilityPlayer capability) {
		this.capability = capability;
		this.type = type;
	}

	public List<AndroidResearchType<?>> getPrerequisites() {
		return type.getPrerequisites();
	}

	public List<AndroidResearchType<?>> getBlocking() {
		return type.getBlocking();
	}

	public List<AndroidResearchType<?>> getBlockedBy() {
		return type.getBlockedBy();
	}

	abstract public void onResearched();
	abstract public void consumeCost();
	abstract public boolean canAfford();

	@Nullable
	public SkinElement getIcon() {
		return null;
	}

	public Component getDisplayName() {
		return type.getDisplayName();
	}

	public List<Component> getTooltip() {
		return List.of(getDisplayName());
	}

	public boolean canResearch() {
		for (var preq : getPrerequisites()) {
			if (!capability.getResearch(preq).isResearched()) {
				return false;
			}
		}

		for (var preq : getBlockedBy()) {
			if (capability.getResearch(preq).isResearched()) {
				return false;
			}
		}

		return canAfford();
	}

	public boolean research(boolean force) {
		if (researched)
			return false;

		if (!force && !canResearch())
			return false;

		consumeCost();
		researched = true;
		onResearched();

		markDirty();

		return true;
	}

	@Nullable
	public Component getIconText() {
		return null;
	}

	public boolean research() {
		return research(false);
	}

	public void researchClient() {
		if (!researched && canAfford() && capability.isAndroid())
			MatteryNetworking.CHANNEL.sendToServer(new AndroidResearchRequestPacket(type));
	}

	@Override
	public CompoundTag serializeNBT() {
		var nbt = new CompoundTag();
		serializeNBT(nbt);
		return nbt;
	}

	public void serializeNBT(CompoundTag tag) {
		tag.putByte("researched", researched ? (byte) 1 : 0);
	}

	public void deserializeNBT(CompoundTag nbt) {
		researched = nbt.getByte("researched") > 0;
	}

}
