package ru.dbotthepony.mc.otm.android.feature;

import net.minecraft.nbt.CompoundTag;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import ru.dbotthepony.mc.otm.android.AndroidFeature;
import ru.dbotthepony.mc.otm.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.capability.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;

import java.math.BigDecimal;

public class AndroidNanobotsRegeneration extends AndroidFeature {
	public AndroidNanobotsRegeneration(AndroidFeatureType<?> type, IAndroidCapability capability) {
		super(type, capability);
	}

	protected int ticks_passed = 0;
	protected int heal_ticks = 0;

	protected static final BigDecimal ENERGY_PER_HITPOINT = new BigDecimal(800);

	protected static final int[] TICKS_BETWEEN_HEAL = new int[] {
		100, // 5 seconds
		80, // 4 seconds
		60, // 3 seconds
		40, // 2 seconds
	};

	@Override
	public void tickServer() {
		var ent = capability.getEntity();

		if (ent.getHealth() > 0 && ent.getHealth() < ent.getMaxHealth()) {
			ticks_passed++;

			var wait_time = heal_ticks >= TICKS_BETWEEN_HEAL.length ? TICKS_BETWEEN_HEAL[TICKS_BETWEEN_HEAL.length - 1] : TICKS_BETWEEN_HEAL[heal_ticks];

			if (ticks_passed > wait_time) {
				var missing = Math.min(1, ent.getMaxHealth() - ent.getHealth());
				var extract = capability.extractEnergyInner(ENERGY_PER_HITPOINT.multiply(new BigDecimal(Float.toString(missing))), false);

				if (extract.compareTo(BigDecimal.ZERO) > 0) {
					heal_ticks = Math.min(heal_ticks + 1, level);
					ent.heal(missing * extract.divide(ENERGY_PER_HITPOINT, MatteryCapability.ROUND_RULES).floatValue());
					ticks_passed = 0;
				}
			}
		} else {
			heal_ticks = 0;
			ticks_passed = 0;
		}
	}

	@Override
	public void onHurt(LivingHurtEvent event) {
		heal_ticks = 0;
		ticks_passed = 0;
	}

	@Override
	public void serializeNBT(CompoundTag tag) {
		super.serializeNBT(tag);
		tag.putInt("heal_ticks", heal_ticks);
		tag.putInt("ticks_passed", ticks_passed);
	}

	@Override
	public void deserializeNBT(CompoundTag tag) {
		super.deserializeNBT(tag);
		heal_ticks = tag.getInt("heal_ticks");
		ticks_passed = tag.getInt("ticks_passed");
	}
}
