package ru.dbotthepony.mc.otm.menu.widget;

import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer;

import java.math.BigDecimal;
import java.util.function.Supplier;

abstract public class StorageGaugeWidget extends GaugeWidget {
	protected Supplier<BigDecimal> value_supplier;
	protected Supplier<BigDecimal> max_value_supplier;

	protected final BigDecimalDataContainer value_container = new BigDecimalDataContainer();
	protected final BigDecimalDataContainer max_value_container = new BigDecimalDataContainer();

	public StorageGaugeWidget(MatteryMenu menu, int x, int y) {
		this(menu, x, y, null, null);
	}

	public StorageGaugeWidget(MatteryMenu menu) {
		this(menu, 0, 0, null, null);
	}

	public StorageGaugeWidget(MatteryMenu menu, int x, int y, Supplier<BigDecimal> value_supplier, Supplier<BigDecimal> max_value_supplier) {
		super(menu, x, y);
		this.value_supplier = value_supplier;
		this.max_value_supplier = max_value_supplier;

		addDataSlots(value_container);
		addDataSlots(max_value_container);
	}

	public StorageGaugeWidget(MatteryMenu menu, Supplier<BigDecimal> value_supplier, Supplier<BigDecimal> max_value_supplier) {
		this(menu, 0, 0, value_supplier, max_value_supplier);
	}

	@Override
	public void updateServer() {
		if (value_supplier == null || max_value_supplier == null)
			return;

		value_container.setDecimal(value_supplier.get());
		max_value_container.setDecimal(max_value_supplier.get());
	}

	@Override
	public float getLevel() {
		return max_value_container.getDecimal().compareTo(BigDecimal.ZERO) == 0 ? 0f : value_container.getDecimal().divide(max_value_container.getDecimal(), MatteryCapability.ROUND_RULES).floatValue();
	}
}
